@@ -385,7 +385,7 @@ int main(int argc,char** argv)
                       segment=NextSegment(OSMSegments,segment,item);
                      }
 
-                   if(node->flags&NODE_TURNRSTRCT)
+                   if(IsTurnRestrictedNode(node))
                      {
                       index_t relindex=FindFirstTurnRelation1(OSMRelations,item);
 
@@ -564,7 +564,7 @@ static void print_turnrelation(Relations *relations,index_t item,Segments *segme
 static void print_head_osm(void)
 {
  printf("<?xml version='1.0' encoding='UTF-8'?>\n");
- printf("<osm version='0.6' generator='JOSM'>\n");
+ printf("<osm version='0.6' generator='Routino'>\n");
 }
 
 
@@ -646,7 +646,7 @@ static void print_segment_osm(Segments *segments,index_t item,Ways *ways)
  if(IsNormalSegment(segment))
     printf("    <tag k='routino:normal' v='yes' />\n");
 
- printf("     <tag k='routino:distance' v='%.3f' />\n",distance_to_km(DISTANCE(segment->distance)));
+ printf("    <tag k='routino:distance' v='%.3f' />\n",distance_to_km(DISTANCE(segment->distance)));
 
  if(way->type & Way_OneWay)
     printf("    <tag k='oneway' v='yes' />\n");
